#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "nbody.h"
#include "mpi.h"

typedef struct vector{
	float x;
	float y;
} vector_t;

// For MPI
// do_step ̃C^[tF[Xɗ͕ςȂ߂ɑϐɂ
// C^[tF[Xςق{͍D܂C
static int rank = -1;
static int size = -1;
static int *assigned_id_min;
static int *assigned_id_max;
static int *assigned_id_length;
static MPI_Datatype MPI_POINT;

static void calc_acceleration(point_t* i, point_t* j, vector_t* accel, state_t* st);
static void do_step(point_t* plots, state_t *st, vector_t* accel);
int main(int argc, char** argv){
	point_t *plots;
	vector_t *accel;
	state_t st;
	int i;
	int nstep;
	int offset = 0;
	
	// MPIAǉ
	MPI_Init(&argc, &argv);
	MPI_Comm_rank( MPI_COMM_WORLD, &rank);
	MPI_Comm_size( MPI_COMM_WORLD, &size);
	//        point_t  MPI_Datatype 쐬
	MPI_Type_vector(5, 1, 1, MPI_FLOAT, &MPI_POINT);
	MPI_Type_commit(&MPI_POINT);
	// ------------
	
		
	// t@Cǂݍ
	init_plots_map(argv[1], &st, &plots);
	// xf[^ۑp̗̈m
	if((accel = (vector_t*) malloc(sizeof(vector_t) * st.nplots)) == NULL){
		perror("malloc accel");
		exit(EXIT_FAILURE);
	}

	// MPIAǉ
	// range : assined_id_min <= id < assined_id_max
	if((assigned_id_min = (int*) calloc(size, sizeof(int))) == NULL){
		perror("malloc assined_id_min");
		exit(EXIT_FAILURE);
	}
	if((assigned_id_max = (int*) calloc(size, sizeof(int))) == NULL){
		perror("malloc assined_id_max");
		exit(EXIT_FAILURE);
	}
	if((assigned_id_length = (int*) calloc(size, sizeof(int))) == NULL){
		perror("malloc assined_id_length");
		exit(EXIT_FAILURE);
	}
	for(i = 0; i < size; i++){
		assigned_id_min[i] = offset;
		assigned_id_length[i] = (int)((st.nplots / size ) + (((st.nplots % size) > i) & 0x1));
		assigned_id_max[i] = assigned_id_min[i] + assigned_id_length[i];
		offset = assigned_id_max[i];
	}
	// ------------
	
	nstep = atoi(argv[3]);

	for(i = 0; i < nstep; i++){
		do_step(plots, &st, accel);		
	}
	
	if(rank == 0)
		write_plots_map(argv[2], &st, plots);
	fini_plots_map(plots);
	
	MPI_Finalize();
	return 1;
}


static void calc_acceleration(point_t* i, point_t* j, vector_t* accel, state_t* st){
	vector_t direction;
	vector_t delta;
	float norm_squr;
	
	//xvZ
	// m * a_vector = GmM / r_norm^2 * r_vector

	// _ԋ
	delta.x = i->x - j->x;
	delta.y = i->y - j->y;
	norm_squr = delta.x * delta.x + delta.y * delta.y;
	// ͂̋yԕ
	direction.x = delta.x / sqrt(norm_squr);
	direction.y = delta.y / sqrt(norm_squr);
	// xZo
	accel->x = (st->gc * j->m / norm_squr) * direction.x;
	accel->y = (st->gc * j->m / norm_squr) * direction.y;

}

static void do_step(point_t* plots, state_t* st, vector_t* accel)
{
	int i, j;
	vector_t accel_work;
	
	//xvZ
	//Jnl Il MPIpɉ
	for(i = assigned_id_min[rank]; i < assigned_id_max[rank]; i++){
		accel[i].x = 0.0;
		accel[i].y = 0.0;
		for(j = 0; j < st->nplots; j++){
			if(i == j) continue;
			calc_acceleration(&(plots[i]), &(plots[j]), &accel_work, st);
			accel[i].x += accel_work.x;
			accel[i].y += accel_work.y;
		}
	}
	//ʒuExvZ
	for(i = assigned_id_min[rank]; i < assigned_id_max[rank]; i++){
		plots[i].vx += accel[i].x * st->delta_t;
		plots[i].vy += accel[i].y * st->delta_t;
		plots[i].x += plots[i].vx * st->delta_t;
		plots[i].y += plots[i].vy * st->delta_t;
	}
	
	//f[^ for MPI
	
	MPI_Allgatherv(&(plots[assigned_id_min[rank]]), assigned_id_length[rank], MPI_POINT, 
			plots, assigned_id_length, assigned_id_min, MPI_POINT, MPI_COMM_WORLD);
	
}

