#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>

#include "nbody.h"
#include "mt19937ar.h"

#define LV0	0
#define LV1	2
#define LV2	4
#define LV3	6
#define LV4	8

char symbol[10] = {
		' ', ' ',
		'.', 'o',
		':', '8',
		'+', '%',
		'*', '#'
};

static float mtrand(float min, float max);

int init_plots_map(char* filename, state_t *st, point_t **plots){
	FILE *fp;
	point_t *tp;
	
	if((fp = fopen(filename, "r")) == NULL){
		perror("fopen");
		exit(EXIT_FAILURE);
	}
	
	fread(st, sizeof(state_t), 1, fp);

	if((tp = (point_t*)malloc(sizeof(point_t) * st->nplots)) == NULL){
//	if(posix_memalign(tp, 16, sizeof(point_t) * st->nplots) != 0){
		perror("posix_memalign init_plots_map");
		exit(EXIT_FAILURE);
	}

	fread(tp, sizeof(point_t), st->nplots, fp);
	*plots = tp;
	return 1;
}

int fini_plots_map(point_t *plots){
	free(plots);
	return 1;
}

void make_plots_map(point_t **plots, state_t *st, int nplots, point_t max, point_t min, int cutoff, int delta_t)
{
	float msum = 0.0;
	point_t *tp;
	int i;
	
	init_genrand(time(NULL));
	
	if((tp = (point_t*) malloc(sizeof(point_t) * nplots)) == NULL){
		perror("malloc");
		exit(EXIT_FAILURE);
	}
	
	for(i = 0; i < nplots; i++){
		tp[i].m = mtrand(min.m, max.m);
		tp[i].x = mtrand(min.x, max.x);
		tp[i].y = mtrand(min.y, max.y);
		tp[i].vx = mtrand(min.vx, max.vx);
		tp[i].vy = mtrand(min.vy, max.vy);
		msum += tp[i].m;
	}
	
	*plots = tp;
	
	st->nplots = nplots;
	st->msum = msum;
	st->xmax = (int)max.x;
	st->ymax = (int)max.y;
	st->xmin = (int)min.x;
	st->ymin = (int)min.y;
	st->cutoff = cutoff;
	st->gc = (float)(cutoff * cutoff) / (float)(max.m * ACCEL_THRESHOLD);
	st->delta_t = delta_t;
}

void write_plots_map(char* filename, state_t *st, point_t *plots){
	FILE *fp;

	if((fp = fopen(filename, "w")) == NULL){
		perror("fopen");
		exit(EXIT_FAILURE);
	}
	
	fwrite(st, sizeof(state_t), 1, fp);
	fwrite(plots, sizeof(point_t), st->nplots, fp);	
}

void print_plots_map(point_t *plots, state_t st, int px, int py){
	field_t * fld;
	int i, x, y, lv = -1;
	float tlv, mave, nave;
	
	float mx;
	float my;
	
	mx = (st.xmax - st.xmin) / px;
	my = (st.ymax - st.ymin) / py;
	mave = st.msum / (float)(px * py);
	nave = (float)(st.nplots) / (float)(px * py);
	
	if((fld = (field_t*)calloc(px * py, sizeof(field_t))) == NULL){
		perror("calloc");
		exit(EXIT_FAILURE);
	}
	
	for(i = 0; i < st.nplots; i++){
		if((plots[i].x > st.xmax) || (plots[i].x < st.xmin))
			continue;
		if((plots[i].y > st.ymax) || (plots[i].y < st.ymin))
			continue;
		x = (int)((plots[i].x + st.ymin) / mx);
		y = (int)((plots[i].y + st.ymin) / my);
		
		fld[y * px + x].nplots++;
		fld[y * px + x].msum += plots[i].m;
	}

	printf("nplots:%d, xrange:%d-%d, yrange:%d-%d, msum:%f\n",
			st.nplots, st.xmin, st.xmax, st.ymin, st.ymax, st.msum);

	for (i = 0; i <= px; i++)
		printf("-");
	printf("|\n|");
	for(i = 0; i < px * py; i++){
		tlv = (float)fld[i].nplots /  nave;
		if(tlv <= 0){
			lv = LV0;
		}else if(tlv <= 0.5){
			lv = LV1;
		}else if(tlv <= 1){
			lv = LV2;
		}else if(tlv <= 1.5){
			lv = LV3;
		}else if (tlv > 1.5){
			lv = LV4;
		}

		if(fld[i].msum > mave)
			lv++;
		
		printf("%c", symbol[lv]);
		if((i % px) == px -1)
			printf("|\n|");
		
	}
	for (i = 0; i <= px; i++)
		printf("-");
	printf("\n");
	free(fld);
	
}

void dump_plots_map(point_t *plots, state_t st){
	int i;
	for(i = 0; i < st.nplots; i++){
		printf("x:%f, y:%f, m:%f, vx:%f, vy:%f\n", plots[i].x, plots[i].y,
				plots[i].m, plots[i].vx, plots[i].vy);
	}
}


static float mtrand(float min, float max){
	//return (float)((max - min) * ((float)rand() / (float)RAND_MAX) + min);
		return (float)((max - min) * genrand_real1() + min);
}


