#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "nbody.h"

#define M_WRITE	0
#define M_READ	1

#define MAX_X	1024.0
#define MAX_Y	1024.0
#define MAX_M	16.0
#define MAX_VX	16.0
#define MAX_VY	16.0

#define MIN_X	0.0
#define MIN_Y	0.0
#define MIN_M	1.0
#define MIN_VX	0.0
#define MIN_VY	0.0

#define PRINT_X	64.0
#define PRINT_Y	32.0

#define CUTOFF	32.0
#define DELTA_T	1.0

int make(int nproc, char* filename);
int print(char* filename);

int main(int argc, char** argv){
	
    int opt;
    int nplots = 0;
    char *filename = NULL;
    int mode = -1;

    while ((opt = getopt(argc, argv, "n:w:r:")) != -1) {
        switch (opt) {
        case 'n':
        	nplots = atoi(optarg);
        	break;
        case 'w':
        	filename = optarg;
        	mode = M_WRITE;
        	break;
        case 'r':
        	filename = optarg;
        	mode = M_READ;
        	break;
        default: /* '?' */
            exit(EXIT_FAILURE);
        }
    }
    
    if(mode == -1)
    	exit(EXIT_FAILURE);
	
    switch (mode){
    case M_WRITE:
    	make(nplots, filename);
    	break;
    case M_READ:
    	print(filename);
    	break;
    }
    
    exit(EXIT_SUCCESS);

}

int make(int nplots, char* filename)
{
	point_t * plots;
	state_t st;
	point_t max, min;
	FILE *fp;

	max.m = MAX_M;
	max.x = MAX_X;
	max.y = MAX_Y;
	max.vx = MAX_VX;
	max.vy = MAX_VY;
	min.m = 1.0;
	min.x = 0.0;
	min.y = 0.0;
	min.vx = 0.0;
	min.vy = 0.0;
		
	make_plots_map(&plots, &st, nplots, max, min, CUTOFF, DELTA_T);
	
	if((fp = fopen(filename, "w")) == NULL){
		perror("fopen");
		exit(EXIT_FAILURE);
	}
	
	fwrite(&st, sizeof(state_t), 1, fp);
	fwrite(plots, sizeof(point_t), nplots, fp);

	fini_plots_map(plots);
	return 1;
	
}

int print(char* filename){
	point_t * plots;
	state_t st;
	
	init_plots_map(filename, &st, &plots);
	
	print_plots_map(plots, st, PRINT_X, PRINT_Y);
	
	return 1;
}




